@@ -22,9 +22,6 @@
 
 #include <QtCore/QByteArray>
 
-#include <ktexteditor/smartrange.h>
-#include <ktexteditor/document.h>
-
 #include <limits>
 
 #include "topducontext.h"
@@ -39,41 +36,54 @@
 #include "indexedstring.h"
 #include "duchainregister.h"
 #include "persistentsymboltable.h"
-#include "repositories/stringrepository.h"
 #include "types/identifiedtype.h"
 #include "types/structuretype.h"
 #include "functiondefinition.h"
 #include "codemodel.h"
 #include "specializationstore.h"
-
-using namespace KTextEditor;
+#include "types/typeutils.h"
+#include "types/typealiastype.h"
+#include "classdeclaration.h"
+#include "repositories/stringrepository.h"
 
 namespace KDevelop
 {
 
-Repositories::StringRepository commentRepository("Comment Repository");
-
 REGISTER_DUCHAIN_ITEM(Declaration);
 
-DeclarationData::DeclarationData() 
-  : m_comment(0), m_isDefinition(false), m_inSymbolTable(false),  
-    m_isTypeAlias(false), m_anonymousInContext(false)
+DeclarationData::DeclarationData()
+  : m_comment(0), m_isDefinition(false), m_inSymbolTable(false), m_isTypeAlias(false),
+    m_anonymousInContext(false), m_isFinal(false), m_alwaysForceDirect(false), m_isAutoDeclaration(false), m_isExplicitlyDeleted(false)
 {
   m_kind = Declaration::Instance;
 }
-  
-DeclarationData::DeclarationData( const DeclarationData& rhs ) : DUChainBaseData(rhs)
+
+DeclarationData::DeclarationData( const DeclarationData& rhs ) : DUChainBaseData(rhs),
+m_internalContext(rhs.m_internalContext),
+m_type(rhs.m_type),
+m_identifier(rhs.m_identifier),
+m_declaration(rhs.m_declaration),
+m_comment(rhs.m_comment),
+m_kind(rhs.m_kind),
+m_isDefinition(rhs.m_isDefinition),
+m_inSymbolTable(rhs.m_inSymbolTable),
+m_isTypeAlias(rhs.m_isTypeAlias),
+m_anonymousInContext(rhs.m_anonymousInContext),
+m_isFinal(rhs.m_isFinal),
+m_alwaysForceDirect(rhs.m_alwaysForceDirect),
+m_isAutoDeclaration(rhs.m_isAutoDeclaration),
+m_isExplicitlyDeleted(rhs.m_isExplicitlyDeleted)
 {
-  m_identifier = rhs.m_identifier;
-  m_declaration = rhs.m_declaration;
-  m_type = rhs.m_type;
-  m_kind = rhs.m_kind;
-  m_isDefinition = rhs.m_isDefinition;
-  m_isTypeAlias = rhs.m_isTypeAlias;
-  m_inSymbolTable = rhs.m_inSymbolTable;
-  m_comment = rhs.m_comment;
-  m_anonymousInContext = rhs.m_anonymousInContext;
-  m_internalContext = rhs.m_internalContext;
+}
+
+///@todo Use reference counting
+static Repositories::StringRepository& commentRepository() {
+    static Repositories::StringRepository commentRepositoryObject("Comment Repository");
+    return commentRepositoryObject;
+}
+
+void initDeclarationRepositories() {
+  commentRepository();
 }
 
 Declaration::Kind Declaration::kind() const {
@@ -94,10 +104,10 @@ bool Declaration::inDUChain() const {
   if( !context() )
     return false;
   TopDUContext* top = topContext();
-  return top && top->inDuChain();
+  return top && top->inDUChain();
 }
 
-Declaration::Declaration( const SimpleRange& range, DUContext* context )
+Declaration::Declaration( const RangeInRevision& range, DUContext* context )
   : DUChainBase(*new DeclarationData, range)
 {
   d_func_dynamic()->setClassId(this);
@@ -117,7 +127,6 @@ uint Declaration::ownIndex() const
 
 Declaration::Declaration(const Declaration& rhs)
   : DUChainBase(*new DeclarationData( *rhs.d_func() )) {
-  setSmartRange(rhs.smartRange(), DocumentRangeObject::DontOwn);
   m_topContext = 0;
   m_context = 0;
   m_indexInTopContext = 0;
@@ -130,7 +139,7 @@ Declaration::Declaration( DeclarationData & dd ) : DUChainBase(dd)
   m_indexInTopContext = 0;
 }
 
-Declaration::Declaration( DeclarationData & dd, const SimpleRange& range )
+Declaration::Declaration( DeclarationData & dd, const RangeInRevision& range )
   : DUChainBase(dd, range)
 {
   m_topContext = 0;
@@ -138,25 +147,31 @@ Declaration::Declaration( DeclarationData & dd, const SimpleRange& range )
   m_indexInTopContext = 0;
 }
 
+bool Declaration::persistentlyDestroying() const
+{
+  TopDUContext* topContext = this->topContext();
+  return !topContext->deleting() || !topContext->isOnDisk();
+}
+
 Declaration::~Declaration()
 {
   uint oldOwnIndex = m_indexInTopContext;
-  
+
   TopDUContext* topContext = this->topContext();
-  
+
   //Only perform the actions when the top-context isn't being deleted, or when it hasn't been stored to disk
-  if(!topContext->deleting() || !topContext->isOnDisk()) {
+  if(persistentlyDestroying()) {
     DUCHAIN_D_DYNAMIC(Declaration);
     // Inserted by the builder after construction has finished.
     if( d->m_internalContext.context() )
       d->m_internalContext.context()->setOwner(0);
-    
+
     if (d->m_inSymbolTable && !d->m_identifier.isEmpty()) {
       QualifiedIdentifier id = qualifiedIdentifier();
       PersistentSymbolTable::self().removeDeclaration(id, this);
       CodeModel::self().removeItem(url(), id);
     }
-    
+
     d->m_inSymbolTable = false;
   }
 
@@ -166,9 +181,9 @@ Declaration::~Declaration()
     if(!topContext->deleting() || !topContext->isOnDisk() || context()->d_func()->isDynamic())
       context()->m_dynamicData->removeDeclaration(this);
   }
-    
+
   clearOwnIndex();
-  
+
   if(!topContext->deleting() || !topContext->isOnDisk()) {
     setContext(0);
 
@@ -183,7 +198,7 @@ QByteArray Declaration::comment() const {
   if(!d->m_comment)
     return 0;
   else
-    return Repositories::arrayFromItem(commentRepository.itemFromIndex(d->m_comment));
+    return Repositories::arrayFromItem(commentRepository().itemFromIndex(d->m_comment));
 }
 
 void Declaration::setComment(const QByteArray& str) {
@@ -191,7 +206,7 @@ void Declaration::setComment(const QByteArray& str) {
   if(str.isEmpty())
     d->m_comment = 0;
   else
-    d->m_comment = commentRepository.index(Repositories::StringRepositoryItemRequest(str, IndexedString::hashString(str, str.length()), str.length()));
+    d->m_comment = commentRepository().index(Repositories::StringRepositoryItemRequest(str, IndexedString::hashString(str, str.length()), str.length()));
 }
 
 void Declaration::setComment(const QString& str) {
@@ -204,7 +219,7 @@ Identifier Declaration::identifier( ) const
   return d_func()->m_identifier.identifier();
 }
 
-IndexedIdentifier Declaration::indexedIdentifier( ) const
+const IndexedIdentifier& Declaration::indexedIdentifier( ) const
 {
   //ENSURE_CAN_READ Commented out for performance reasons
   return d_func()->m_identifier;
@@ -263,11 +278,11 @@ Declaration* IndexedDeclaration::declaration() const {
 void Declaration::rebuildDynamicData(DUContext* parent, uint ownIndex)
 {
   DUChainBase::rebuildDynamicData(parent, ownIndex);
-  
+
   m_context = parent;
   m_topContext = parent->topContext();
   m_indexInTopContext = ownIndex;
-  
+
   parent->m_dynamicData->addDeclarationToHash(d_func()->m_identifier.identifier(), this);
 }
 
@@ -276,14 +291,14 @@ void Declaration::setIdentifier(const Identifier& identifier)
   ENSURE_CAN_WRITE
   DUCHAIN_D_DYNAMIC(Declaration);
   bool wasInSymbolTable = d->m_inSymbolTable;
-  
+
   setInSymbolTable(false);
-    
+
   if( m_context && !d->m_anonymousInContext )
     m_context->changingIdentifier( this, d->m_identifier, identifier );
 
   d->m_identifier = identifier;
-  
+
   setInSymbolTable(wasInSymbolTable);
   //DUChain::declarationChanged(this, DUChainObserver::Change, DUChainObserver::Identifier);
 }
@@ -296,7 +311,7 @@ IndexedType Declaration::indexedType() const
 AbstractType::Ptr Declaration::abstractType( ) const
 {
   //ENSURE_CAN_READ Commented out for performance reasons
-  return d_func()->m_type.type();
+  return d_func()->m_type.abstractType();
 }
 
 void Declaration::setAbstractType(AbstractType::Ptr type)
@@ -314,8 +329,10 @@ void Declaration::setAbstractType(AbstractType::Ptr type)
     //DUChain::declarationChanged(this, DUChainObserver::Addition, DUChainObserver::DataType);
 }
 
-Declaration* Declaration::specialize(uint /*specialization*/, const TopDUContext* /*topContext*/, int /*upDistance*/)
+Declaration* Declaration::specialize(IndexedInstantiationInformation /*specialization*/, const TopDUContext* topContext, int /*upDistance*/)
 {
+  if(!topContext)
+    return 0;
   return this;
 }
 
@@ -357,7 +374,7 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 {
   Q_ASSERT(!context || context->topContext());
   ///@todo re-enable. In C++ support we need a short window to put visible declarations into template contexts
-  if(!specialization()) {
+  if(!specialization().index()) {
     //problem: specialization() doesn't work during destructor
 //     ENSURE_CAN_WRITE
   }
@@ -366,8 +383,8 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 
   //We don't need to clear, because it's not allowed to move from one top-context into another
 //   clearOwnIndex();
-  
-  
+
+
   DUCHAIN_D_DYNAMIC(Declaration);
   if (m_context && context) {
     Q_ASSERT(m_context->topContext() == context->topContext());
@@ -384,14 +401,14 @@ void Declaration::setContext(DUContext* context, bool anonymous)
     m_topContext = context->topContext();
   else
     m_topContext = 0;
-  
+
   d->m_anonymousInContext = anonymous;
   m_context = context;
 
   if (context) {
     if(!m_indexInTopContext)
       allocateOwnIndex();
-    
+
     if(!d->m_anonymousInContext) {
       context->m_dynamicData->addDeclaration(this);
       //DUChain::declarationChanged(this, DUChainObserver::Addition, DUChainObserver::Context, m_context);
@@ -403,14 +420,14 @@ void Declaration::setContext(DUContext* context, bool anonymous)
 }
 
 void Declaration::clearOwnIndex() {
-  
+
   if(!m_indexInTopContext)
     return;
-  
+
   if(!context() || (!d_func()->m_anonymousInContext && !context()->isAnonymous())) {
     ENSURE_CAN_WRITE
   }
-  
+
   if(m_indexInTopContext) {
     Q_ASSERT(m_topContext);
     m_topContext->m_dynamicData->clearDeclarationIndex(this);
@@ -419,7 +436,7 @@ void Declaration::clearOwnIndex() {
 }
 
 void Declaration::allocateOwnIndex() {
-  
+
   ///@todo Fix multithreading stuff with template instantiation, preferably using some internal mutexes
 //   if(context() && (!context()->isAnonymous() && !d_func()->m_anonymousInContext)) {
 //     ENSURE_CAN_WRITE
@@ -429,10 +446,10 @@ void Declaration::allocateOwnIndex() {
 
   m_indexInTopContext = m_topContext->m_dynamicData->allocateDeclarationIndex(this, d_func()->m_anonymousInContext || !context() || context()->isAnonymous());
   Q_ASSERT(m_indexInTopContext);
-  
+
   if(!m_topContext->m_dynamicData->getDeclarationForIndex(m_indexInTopContext))
     kFatal() << "Could not re-retrieve declaration" << "index:" << m_indexInTopContext;
-  
+
 }
 
 const Declaration* Declaration::logicalDeclaration(const TopDUContext* topContext) const {
@@ -468,10 +485,18 @@ DUContext * Declaration::logicalInternalContext(const TopDUContext* topContext)
 
   if( d_func()->m_isTypeAlias ) {
     ///If this is a type-alias, return the internal context of the actual type.
-    AbstractType::Ptr t = abstractType();
-    IdentifiedType* idType = dynamic_cast<IdentifiedType*>(t.unsafeData());
-    if( idType && idType->declaration(topContext) && idType->declaration(topContext) != this )
-      return idType->declaration(topContext)->logicalInternalContext( topContext );
+    TypeAliasType::Ptr t = type<TypeAliasType>();
+    if(t) {
+      AbstractType::Ptr target = t->type();
+      
+      IdentifiedType* idType = dynamic_cast<IdentifiedType*>(target.unsafeData());
+      if( idType ) {
+        Declaration* decl = idType->declaration(topContext);
+        if(decl && decl != this) {
+          return decl->logicalInternalContext( topContext );
+        }
+      }
+    }
   }
 
   return internalContext();
@@ -498,15 +523,15 @@ void Declaration::setInternalContext(DUContext* context)
     m_topContext = context->topContext();
     allocateOwnIndex();
   }
-  
+
   DUContext* oldInternalContext = d->m_internalContext.context();
-  
+
   d->m_internalContext = context;
 
   //Q_ASSERT( !oldInternalContext || oldInternalContext->owner() == this );
   if( oldInternalContext && oldInternalContext->owner() == this )
     oldInternalContext->setOwner(0);
-  
+
 
   if( context )
     context->setOwner(this);
@@ -545,6 +570,46 @@ void Declaration::setDeclarationIsDefinition(bool dd)
 //   }
 }
 
+bool Declaration::isAutoDeclaration() const
+{
+  return d_func()->m_isAutoDeclaration;
+}
+
+void Declaration::setAutoDeclaration(bool _auto)
+{
+  d_func_dynamic()->m_isAutoDeclaration = _auto;
+}
+
+bool Declaration::isFinal() const
+{
+  return d_func()->m_isFinal;
+}
+
+void Declaration::setFinal(bool final)
+{
+  d_func_dynamic()->m_isFinal = final;
+}
+
+bool Declaration::alwaysForceDirect() const
+{
+  return d_func()->m_alwaysForceDirect;
+}
+
+void Declaration::setAlwaysForceDirect(bool direct)
+{
+  d_func_dynamic()->m_alwaysForceDirect = direct;
+}
+
+bool Declaration::isExplicitlyDeleted() const
+{
+  return d_func()->m_isExplicitlyDeleted;
+}
+
+void Declaration::setExplicitlyDeleted(bool deleted)
+{
+  d_func_dynamic()->m_isExplicitlyDeleted = deleted;
+}
+
 ///@todo see whether it would be useful to create an own TypeAliasDeclaration sub-class for this
 bool Declaration::isTypeAlias() const {
   DUCHAIN_D(Declaration);
@@ -556,15 +621,15 @@ void Declaration::setIsTypeAlias(bool isTypeAlias) {
   d->m_isTypeAlias = isTypeAlias;
 }
 
-uint Declaration::specialization() const {
-  return 0;
+IndexedInstantiationInformation Declaration::specialization() const {
+  return IndexedInstantiationInformation();
 }
 
 void Declaration::activateSpecialization()
 {
-  if(specialization()) {
+  if(specialization().index()) {
     DeclarationId baseId(id());
-    baseId.setSpecialization(0);
+    baseId.setSpecialization(IndexedInstantiationInformation());
     SpecializationStore::self().set(baseId, specialization());
   }
 }
@@ -572,7 +637,7 @@ void Declaration::activateSpecialization()
 DeclarationId Declaration::id(bool forceDirect) const
 {
   ENSURE_CAN_READ
-  if(inSymbolTable() && !forceDirect)
+  if(inSymbolTable() && !forceDirect && !alwaysForceDirect())
     return DeclarationId(qualifiedIdentifier(), additionalIdentity(), specialization());
   else
     return DeclarationId(IndexedDeclaration(const_cast<Declaration*>(this)), specialization());
@@ -586,23 +651,26 @@ bool Declaration::inSymbolTable() const
 
 CodeModelItem::Kind kindForDeclaration(Declaration* decl) {
   CodeModelItem::Kind kind = CodeModelItem::Unknown;
-  
+
   if(decl->kind() == Declaration::Namespace)
     return CodeModelItem::Namespace;
-  
+
   if(decl->isFunctionDeclaration()) {
     kind = CodeModelItem::Function;
   }
-  
-  if(decl->kind() == Declaration::Type && decl->type<StructureType>())
+
+  if(decl->kind() == Declaration::Type && (decl->type<StructureType>() || dynamic_cast<ClassDeclaration*>(decl)))
     kind = CodeModelItem::Class;
-  
+
   if(kind == CodeModelItem::Unknown && decl->kind() == Declaration::Instance)
     kind = CodeModelItem::Variable;
-  
+
   if(decl->isForwardDeclaration())
     kind = (CodeModelItem::Kind)(kind | CodeModelItem::ForwardDeclaration);
-  
+
+  if ( decl->context() && decl->context()->type() == DUContext::Class )
+    kind = (CodeModelItem::Kind)(kind | CodeModelItem::ClassMember);
+
   return kind;
 }
 
@@ -622,14 +690,14 @@ void Declaration::setInSymbolTable(bool inSymbolTable)
     if(!d->m_inSymbolTable && inSymbolTable) {
       QualifiedIdentifier id(qualifiedIdentifier());
       PersistentSymbolTable::self().addDeclaration(id, this);
-      
+
       CodeModel::self().addItem(url(), id, kindForDeclaration(this));
     }
-  
+
     else if(d->m_inSymbolTable && !inSymbolTable) {
       QualifiedIdentifier id(qualifiedIdentifier());
       PersistentSymbolTable::self().removeDeclaration(id, this);
-      
+
       CodeModel::self().removeItem(url(), id);
     }
   }
@@ -681,37 +749,75 @@ bool Declaration::equalQualifiedIdentifier(const Declaration* rhs) const {
   DUCHAIN_D(Declaration);
   if(d->m_identifier != rhs->d_func()->m_identifier)
     return false;
-  
+
   return m_context->equalScopeIdentifier(m_context);
 }
 
-QList<KTextEditor::SmartRange*> Declaration::smartUses() const
+QMap<IndexedString, QList<RangeInRevision> > Declaration::uses() const
 {
-  Q_ASSERT(topContext());
   ENSURE_CAN_READ
-  QSet<KTextEditor::SmartRange*> tempUses;
+  QMap<IndexedString, QMap<RangeInRevision, bool> > tempUses;
+
   //First, search for uses within the own context
   {
-    foreach(KTextEditor::SmartRange* range, allSmartUses(topContext(), const_cast<Declaration*>(this)))
-      tempUses.insert(range);
+    QMap<RangeInRevision, bool>& ranges(tempUses[topContext()->url()]);
+    foreach(const RangeInRevision& range, allUses(topContext(), const_cast<Declaration*>(this)))
+      ranges[range] = true;
   }
 
   KDevVarLengthArray<IndexedTopDUContext> useContexts = DUChain::uses()->uses(id());
 
-  FOREACH_ARRAY(IndexedTopDUContext indexedContext, useContexts) {
-    if(!indexedContext.isLoaded())
-      continue;
+  FOREACH_ARRAY(const IndexedTopDUContext& indexedContext, useContexts) {
     TopDUContext* context = indexedContext.data();
     if(context) {
-      foreach(KTextEditor::SmartRange* range, allSmartUses(context, const_cast<Declaration*>(this)))
-        tempUses.insert(range);
+      QMap<RangeInRevision, bool>& ranges(tempUses[context->url()]);
+      foreach(const RangeInRevision& range, allUses(context, const_cast<Declaration*>(this)))
+        ranges[range] = true;
     }
   }
 
-  return tempUses.toList();
+  QMap<IndexedString, QList<RangeInRevision> > ret;
+
+  for(QMap<IndexedString, QMap<RangeInRevision, bool> >::const_iterator it = tempUses.constBegin(); it != tempUses.constEnd(); ++it) {
+    if(!(*it).isEmpty()) {
+      QList<RangeInRevision>& list(ret[it.key()]);
+      for(QMap<RangeInRevision, bool>::const_iterator it2 = (*it).constBegin(); it2 != (*it).constEnd(); ++it2)
+        list << it2.key();
+    }
+  }
+  return ret;
 }
 
-QMap<IndexedString, QList<SimpleRange> > Declaration::uses() const
+bool hasDeclarationUse(DUContext* context, int declIdx)
+{
+  bool ret=false;
+  int usescount=context->usesCount();
+  const Use* uses=context->uses();
+  
+  for(int i=0; !ret && i<usescount; ++i) {
+    ret = uses[i].m_declarationIndex==declIdx;
+  }
+  
+  foreach(DUContext* child, context->childContexts()) {
+    ret = ret || hasDeclarationUse(child, declIdx);
+    if(ret)
+      break;
+  }
+  
+  return ret;
+}
+
+bool Declaration::hasUses() const
+{
+  ENSURE_CAN_READ
+  int idx = topContext()->indexForUsedDeclaration(const_cast<Declaration*>(this), false);
+  bool ret = idx != std::numeric_limits<int>::max() && (idx>=0 || hasDeclarationUse(topContext(), idx)); //hasLocalUses
+  ret =  ret || DUChain::uses()->hasUses(id()); //hasUsesInOtherContexts 
+  
+  return ret;
+}
+
+QMap<IndexedString, QList<SimpleRange> > Declaration::usesCurrentRevision() const
 {
   ENSURE_CAN_READ
   QMap<IndexedString, QMap<SimpleRange, bool> > tempUses;
@@ -719,27 +825,29 @@ QMap<IndexedString, QList<SimpleRange> > Declaration::uses() const
   //First, search for uses within the own context
   {
     QMap<SimpleRange, bool>& ranges(tempUses[topContext()->url()]);
-    foreach(const SimpleRange& range, allUses(topContext(), const_cast<Declaration*>(this)))
-      ranges[range] = true;
+    foreach(const RangeInRevision& range, allUses(topContext(), const_cast<Declaration*>(this)))
+    {
+      ranges[topContext()->transformFromLocalRevision(range)] = true;
+    }
   }
 
   KDevVarLengthArray<IndexedTopDUContext> useContexts = DUChain::uses()->uses(id());
 
-  FOREACH_ARRAY(IndexedTopDUContext indexedContext, useContexts) {
+  FOREACH_ARRAY(const IndexedTopDUContext& indexedContext, useContexts) {
     TopDUContext* context = indexedContext.data();
     if(context) {
       QMap<SimpleRange, bool>& ranges(tempUses[context->url()]);
-      foreach(const SimpleRange& range, allUses(context, const_cast<Declaration*>(this)))
-        ranges[range] = true;
+      foreach(const RangeInRevision& range, allUses(context, const_cast<Declaration*>(this)))
+        ranges[context->transformFromLocalRevision(range)] = true;
     }
   }
 
-  QMap<IndexedString, QList<SimpleRange> > ret;
+QMap<IndexedString, QList<SimpleRange> > ret;
 
-  for(QMap<IndexedString, QMap<SimpleRange, bool> >::const_iterator it = tempUses.begin(); it != tempUses.end(); ++it) {
+  for(QMap<IndexedString, QMap<SimpleRange, bool> >::const_iterator it = tempUses.constBegin(); it != tempUses.constEnd(); ++it) {
     if(!(*it).isEmpty()) {
       QList<SimpleRange>& list(ret[it.key()]);
-      for(QMap<SimpleRange, bool>::const_iterator it2 = (*it).begin(); it2 != (*it).end(); ++it2)
+      for(QMap<SimpleRange, bool>::const_iterator it2 = (*it).constBegin(); it2 != (*it).constEnd(); ++it2)
         list << it2.key();
     }
   }
