@@ -64,10 +64,6 @@
 #include "referencecounting.h"
 #include "importers.h"
 
-Q_DECLARE_METATYPE(KDevelop::IndexedString)
-Q_DECLARE_METATYPE(KDevelop::IndexedTopDUContext)
-Q_DECLARE_METATYPE(KDevelop::ReferencedTopDUContext)
-
 namespace {
 //Additional "soft" cleanup steps that are done before the actual cleanup.
 //During "soft" cleanup, the consistency is not guaranteed. The repository is
@@ -1338,7 +1334,7 @@ TopDUContext* DUChain::chainForDocument(const KDevelop::IndexedString& document,
     }
 
   foreach(const ParsingEnvironmentFilePointer &file, list)
-    if((proxyContext == file->isProxyContext())) {
+    if(proxyContext == file->isProxyContext()) {
       return file->topContext();
     }
 
@@ -1393,7 +1389,10 @@ ParsingEnvironmentFilePointer DUChain::environmentFileForDocument( const Indexed
 
   QList< ParsingEnvironmentFilePointer>::const_iterator it = list.constBegin();
   while(it != list.constEnd()) {
-    if(*it && ((*it)->isProxyContext() == proxyContext) && (*it)->matchEnvironment(environment)) {
+    if(*it && ((*it)->isProxyContext() == proxyContext) && (*it)->matchEnvironment(environment) &&
+      // Verify that the environment-file and its top-context are "good": The top-context must exist,
+      // and there must be a content-context associated to the proxy-context.
+      (*it)->topContext() && (!proxyContext || DUChainUtils::contentContextFromProxyContext((*it)->topContext())) ) {
       return *it;
     }
     ++it;
@@ -1475,7 +1474,7 @@ void DUChain::documentActivated(KDevelop::IDocument* doc)
   //If yes, update it.
   DUChainReadLocker lock( DUChain::lock() );
   QMutexLocker l(&sdDUChainPrivate->m_chainsMutex);
-  TopDUContext* ctx = DUChainUtils::standardContextForUrl(doc->url());
+  TopDUContext* ctx = DUChainUtils::standardContextForUrl(doc->url(), true);
   if(ctx && ctx->parsingEnvironmentFile())
     if(ctx->parsingEnvironmentFile()->needsUpdate())
       ICore::self()->languageController()->backgroundParser()->addDocument(doc->url());
@@ -1525,6 +1524,8 @@ void DUChain::documentLoadedPrepare(KDevelop::IDocument* doc)
             allImportsLoaded = false;
 
         if(allImportsLoaded) {
+          l.unlock();
+          lock.unlock();
           foreach( KDevelop::ILanguage* language, languages)
             if(language->languageSupport() && language->languageSupport()->codeHighlighting())
               language->languageSupport()->codeHighlighting()->highlightDUChain(standardContext);
@@ -1644,26 +1645,6 @@ void DUChain::emitDeclarationSelected(DeclarationPointer decl) {
   emit declarationSelected(decl);
 }
 
-TopDUContext* contentContextFromProxyContext(TopDUContext* top)
-{
-  if(!top)
-    return 0;
-  if(top->parsingEnvironmentFile() && top->parsingEnvironmentFile()->isProxyContext()) {
-    if(!top->importedParentContexts().isEmpty())
-    {
-      TopDUContext* ret = top->importedParentContexts()[0].context(0)->topContext();
-      if(ret->url() != top->url())
-        kDebug() << "url-mismatch between content and proxy:" << top->url().toUrl() << ret->url().toUrl();
-      if(ret->url() == top->url() && !ret->parsingEnvironmentFile()->isProxyContext())
-        return ret;
-    }
-    else {
-      kDebug() << "Proxy-context imports no content-context";
-    }
-  } else
-    return top;
-  return 0;
-}
 KDevelop::ReferencedTopDUContext DUChain::waitForUpdate(const KDevelop::IndexedString& document, KDevelop::TopDUContext::Features minFeatures, bool proxyContext) {
   Q_ASSERT(!lock()->currentThreadHasReadLock() && !lock()->currentThreadHasWriteLock());
 
@@ -1680,17 +1661,13 @@ KDevelop::ReferencedTopDUContext DUChain::waitForUpdate(const KDevelop::IndexedS
 //   waiter.m_waitMutex.lock();
 //   waiter.m_dataMutex.unlock();
   while(!waiter.m_ready) {
-    TemporarilyReleaseForegroundLock release;
-    ///@todo When we don't do this, the backgroundparser doesn't process anything.
-    ///      The background-parser should be moved into an own thread, so we wouldn't need to do this.
     QApplication::processEvents();
-    usleep(10000);
-//     waiter.m_wait.wait(&waiter.m_waitMutex, 10);
+    usleep(1000);
   }
 
   if(!proxyContext) {
     DUChainReadLocker readLock(DUChain::lock());
-    return contentContextFromProxyContext(waiter.m_topContext);
+    return DUChainUtils::contentContextFromProxyContext(waiter.m_topContext);
   }
 
   return waiter.m_topContext;
