@@ -92,7 +92,7 @@ NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg)
     setBehaviours(KDevelop::IOutputView::AllowUserClose | KDevelop::IOutputView::AutoScroll);
     setModel( new KDevelop::OutputModel(), KDevelop::IOutputView::TakeOwnership );
     
-    connect( lineMaker, SIGNAL(receivedStdoutLines(const QStringList&)), model(), SLOT(appendLines(QStringList)) );
+    connect( lineMaker, SIGNAL(receivedStdoutLines(QStringList)), model(), SLOT(appendLines(QStringList)) );
     connect( proc, SIGNAL(error(QProcess::ProcessError)), SLOT(processError(QProcess::ProcessError)) );
     connect( proc, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(processFinished(int,QProcess::ExitStatus)) );
 
@@ -111,7 +111,20 @@ NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg)
     
     proc->setOutputChannelMode(KProcess::MergedChannels);
     
-    proc->setProgram( executable.toLocalFile(), arguments );
+    if (iface->useTerminal(cfg)) {
+        QStringList args = KShell::splitArgs(iface->terminal(cfg));
+        for (QStringList::iterator it = args.begin(); it != args.end(); ++it) {
+            if (*it == "%exe") {
+                *it = KShell::quoteArg(executable.toLocalFile());
+            } else if (*it == "%workdir") {
+                *it = KShell::quoteArg(wc.toLocalFile());
+            }
+        }
+        args.append( arguments );
+        proc->setProgram( args );
+    } else {
+        proc->setProgram( executable.toLocalFile(), arguments );
+    }
     
     setObjectName(cfg->name());
 }
@@ -123,7 +136,7 @@ void NativeAppJob::start()
     if( proc )
     {
         startOutput();
-        appendLine( i18n("Starting: %1", proc->property("executable").toString() ) );
+        appendLine( i18n("Starting: %1", proc->program().join(" ") ) );
         proc->start();
     } else
     {
