@@ -1,7 +1,8 @@
 /*
  * KDevelop Problem Reporter
  *
- * Copyright (c) 2008 Hamish Rodda <rodda@kde.org>
+ * Copyright 2008 Hamish Rodda <rodda@kde.org>
+ * Copyright 2008-2009 David Nolden <david.nolden.kdevelop@art-master.de>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Library General Public License as
@@ -22,28 +23,35 @@
 #ifndef PROBLEM_HIGHLIGHT_H
 #define PROBLEM_HIGHLIGHT_H
 
-#include <KTextEditor/SmartRangeWatcher>
 
 #include <language/interfaces/iproblem.h>
+#include <qpointer.h>
+#include <ktexteditor/movingrange.h>
 
-class ProblemHighlighter : public KTextEditor::SmartRangeWatcher
+class ProblemHighlighter : public QObject
 {
+    Q_OBJECT
 public:
     ProblemHighlighter(KTextEditor::Document* document);
     virtual ~ProblemHighlighter();
 
     void setProblems(const QList<KDevelop::ProblemPointer>& problems);
 
+private slots:
+    void viewCreated(KTextEditor::Document*,KTextEditor::View*);
+    void textHintRequested(const KTextEditor::Cursor&, QString&);
+    void aboutToInvalidateMovingInterfaceContent();
+    void aboutToRemoveText(const KTextEditor::Range& range);
+    void documentReloaded();
+
 private:
-    /// Detect and respond to a highlighted range being deleted
-    virtual void rangeDeleted(KTextEditor::SmartRange *range);
-    /// Hide the error when the text is changed, it's likely to be different on the next parse
-    virtual void rangeContentsChanged(KTextEditor::SmartRange* range);
+    QPointer<KTextEditor::Document> m_document;
+    QList<KTextEditor::MovingRange*> m_topHLRanges;
+    QList<KDevelop::ProblemPointer> m_problems;
+    QMap<KTextEditor::MovingRange*, KDevelop::ProblemPointer> m_problemsForRanges;
 
-    void clearHighlights();
-
-    KTextEditor::Document* m_document;
-    QList<KTextEditor::SmartRange*> m_topHLRanges;
+public slots:
+    void settingsChanged();
 };
 
 #endif // PROBLEM_HIGHLIGHT_H
