@@ -88,8 +88,7 @@ private:
 void ProjectModelTest::initTestCase()
 {
     KDevelop::AutoTestShell::init();
-    KDevelop::TestCore* core = new KDevelop::TestCore();
-    core->initialize(KDevelop::Core::NoUi);
+    KDevelop::TestCore::initialize(KDevelop::Core::NoUi);
 
     qRegisterMetaType<QModelIndex>("QModelIndex");
     model = new ProjectModel( this );
@@ -104,6 +103,7 @@ void ProjectModelTest::init()
 void ProjectModelTest::cleanupTestCase()
 {
     delete model;
+    KDevelop::TestCore::shutdown();
 }
 
 void ProjectModelTest::testCreateFileSystemItems()
@@ -334,7 +334,7 @@ void ProjectModelTest::testItemSanity()
     // Check that model is properly emitting data-changes
     model->appendRow( parent );
     QCOMPARE( parent->index(), model->index(0, 0, QModelIndex()) );
-    QSignalSpy s( model, SIGNAL(dataChanged( const QModelIndex&, const QModelIndex& )) );
+    QSignalSpy s( model, SIGNAL(dataChanged(QModelIndex,QModelIndex)) );
     parent->setUrl( KUrl("file:///newtest") );
     QCOMPARE( s.count(), 1 );
     QCOMPARE( model->data( parent->index() ).toString(), QString("newtest") );
@@ -369,25 +369,34 @@ void ProjectModelTest::testRename()
     QFETCH( QString, expectedItemText );
     QFETCH( int, expectedRenameCode );
 
+    DummyProject* proj = new DummyProject( "DummyProject", 0 );
+    ProjectFolderItem* rootItem = new ProjectFolderItem( proj, KUrl("file:///dummyprojectfolder"), 0);
+    proj->setProjectItem( rootItem );
+    model->appendRow( rootItem );
+
+    new ProjectFileItem(proj, KUrl("existing"), rootItem);
+
     ProjectBaseItem* item = 0;
     if( itemType == ProjectBaseItem::Target ) {
-        item = new ProjectTargetItem( 0, itemText );
+        item = new ProjectTargetItem( proj, itemText, rootItem );
     } else if( itemType == ProjectBaseItem::File ) {
-        item = new ProjectFileItem( 0, KUrl(itemText) );
+        item = new ProjectFileItem( proj, KUrl(itemText), rootItem );
     } else if( itemType == ProjectBaseItem::Folder ) {
-        item = new ProjectFolderItem( 0, KUrl(itemText) );
+        item = new ProjectFolderItem( proj, KUrl(itemText), rootItem );
     } else if( itemType == ProjectBaseItem::BuildFolder ) {
-        item = new ProjectBuildFolderItem( 0, KUrl(itemText) );
+        item = new ProjectBuildFolderItem( proj, KUrl(itemText), rootItem );
     }
     Q_ASSERT( item );
-    
-    model->appendRow( item );
-    QSignalSpy s( model, SIGNAL(dataChanged(const QModelIndex&, const QModelIndex&)) );
+
+    QCOMPARE(item->model(), model);
+    QSignalSpy s( model, SIGNAL(dataChanged(QModelIndex,QModelIndex)) );
     ProjectBaseItem::RenameStatus stat = item->rename( newName );
     QCOMPARE( (int)stat, expectedRenameCode );
     if( datachangesignal ) {
         QCOMPARE( s.count(), 1 );
         QCOMPARE( qvariant_cast<QModelIndex>( s.takeFirst().at(0) ), item->index() );
+    } else {
+        QCOMPARE( s.count(), 0 );
     }
     QCOMPARE( item->text(), expectedItemText );
 }
@@ -436,7 +445,7 @@ void ProjectModelTest::testRename_data()
     QTest::newRow("ExistingFileError")
     << (int)ProjectBaseItem::Folder
     << QString::fromLatin1("mynew")
-    << QString::fromLatin1("bin")
+    << QString::fromLatin1("existing")
     << false
     << QString::fromLatin1("mynew")
     << (int)ProjectBaseItem::ExistingItemSameName;
@@ -468,8 +477,8 @@ void ProjectModelTest::testAddItemInThread()
     model->appendRow( root );
     AddItemThread t( root );
     SignalReceiver check( model );
-    connect( model, SIGNAL(rowsInserted( const QModelIndex&, int, int )), &check, SLOT(rowsInserted(const QModelIndex&, int, int)), Qt::DirectConnection );
-    KDevelop::KDevSignalSpy spy( &t, SIGNAL( addedItems() ), Qt::QueuedConnection );
+    connect( model, SIGNAL(rowsInserted(QModelIndex,int,int)), &check, SLOT(rowsInserted(QModelIndex,int,int)), Qt::DirectConnection );
+    KDevelop::KDevSignalSpy spy( &t, SIGNAL(addedItems()), Qt::QueuedConnection );
     t.start();
     QVERIFY(spy.wait( 10000 ));
     QCOMPARE( qApp->thread(), check.threadOfSignalEmission() );
@@ -491,6 +500,45 @@ void ProjectModelTest::testDeleteLots()
     modelTest = new ModelTest( model, this );
 }
 
+void ProjectModelTest::testItemsForUrl()
+{
+    QFETCH(KUrl, url);
+    QFETCH(ProjectBaseItem*, root);
+    QFETCH(int, matches);
+
+    model->appendRow(root);
+
+    QList< ProjectBaseItem* > items = model->itemsForUrl(url);
+    QCOMPARE(items.size(), matches);
+    foreach(ProjectBaseItem* item, items) {
+        QVERIFY(item->url().equals(url, KUrl::CompareWithoutTrailingSlash));
+    }
+
+    model->clear();
+}
+
+void ProjectModelTest::testItemsForUrl_data()
+{
+    QTest::addColumn<KUrl>("url");
+    QTest::addColumn<ProjectBaseItem*>("root");
+    QTest::addColumn<int>("matches");
+
+    {
+        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:////tmp/"));
+        ProjectFileItem* file = new ProjectFileItem(0, KUrl(root->url(), "a"), root);
+        QTest::newRow("find one") << file->url() << static_cast<ProjectBaseItem*>(root) << 1;
+    }
+
+    {
+        ProjectFolderItem* root = new ProjectFolderItem(0, KUrl("file:////tmp/"));
+        ProjectFolderItem* folder = new ProjectFolderItem(0, KUrl(root->url(), "a"), root);
+        ProjectFileItem* file = new ProjectFileItem(0, KUrl(folder->url(), "foo"), folder);
+        ProjectTargetItem* target = new ProjectTargetItem(0, "b", root);
+        ProjectFileItem* file2 = new ProjectFileItem(0, file->url(), target);
+        QTest::newRow("find two") << file->url() << static_cast<ProjectBaseItem*>(root) << 2;
+    }
+}
+
 QTEST_KDEMAIN( ProjectModelTest, GUI)
 #include "projectmodeltest.moc"
 #include "moc_projectmodeltest.cpp"
