@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright 2006 Adam Treat <treat@kde.org>                         *
- *   Copyright 2007 Alexander Dymo <adymo@kdevelop.org>             *
+ *   Copyright 2006 Adam Treat <treat@kde.org>                             *
+ *   Copyright 2007 Alexander Dymo <adymo@kdevelop.org>                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -272,28 +272,50 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
 
     KMimeType::Ptr mimeType;
     
-    if(!extension.isEmpty())
+    if(!extension.isEmpty()) {
         // If we have recognized a file extension, allow using the file-contents
         // to look up the type. We will cache it after all.
         mimeType = KMimeType::findByUrl(url);
-    else
+    } else {
         // If we have not recognized a file extension, do not allow using the file-contents
         // to look up the type. We cannot cache the result, and thus we might end up reading
         // the contents of every single file, which can make the application very unresponsive.
         mimeType = KMimeType::findByUrl(url, 0, false, true);
 
-    LanguageCache::ConstIterator it = d->languageCache.constFind(mimeType->name());
+        if (mimeType->isDefault()) {
+            // ask the document controller about a more concrete mimetype
+            IDocument* doc = ICore::self()->documentController()->documentForUrl(url);
+            if (doc) {
+                mimeType = doc->mimeType();
+            }
+        }
+    }
+
+    languages = languagesForMimetype(mimeType->name());
+
+    if(!extension.isEmpty())
+        d->fileExtensionCache.insert(extension, languages);
+
+    return languages;
+}
+
+QList<ILanguage*> LanguageController::languagesForMimetype(const QString& mimetype)
+{
+    QMutexLocker lock(&d->dataMutex);
+
+    QList<ILanguage*> languages;
+    LanguageCache::ConstIterator it = d->languageCache.constFind(mimetype);
     if (it != d->languageCache.constEnd()) {
         languages = it.value();
     } else {
         QStringList constraints;
-        constraints << QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimeType->name());
+        constraints << QString("'%1' in [X-KDevelop-SupportedMimeTypes]").arg(mimetype);
         QList<IPlugin*> supports = Core::self()->pluginController()->
             allPluginsForExtension("ILanguageSupport", constraints);
 
         if (supports.isEmpty()) {
-            kDebug(9505) << "no languages for mimetype:" << mimeType->name();
-            d->languageCache.insert(mimeType->name(), QList<ILanguage*>());
+            kDebug(9505) << "no languages for mimetype:" << mimetype;
+            d->languageCache.insert(mimetype, QList<ILanguage*>());
         } else {
             foreach (IPlugin *support, supports) {
                 ILanguageSupport* languageSupport = support->extension<ILanguageSupport>();
@@ -303,13 +325,25 @@ QList<ILanguage*> LanguageController::languagesForUrl(const KUrl &url)
             }
         }
     }
-
-    if(!extension.isEmpty())
-        d->fileExtensionCache.insert(extension, languages);
-
     return languages;
 }
 
+QList<QString> LanguageController::mimetypesForLanguageName(const QString& languageName)
+{
+    QMutexLocker lock(&d->dataMutex);
+
+    QList<QString> mimetypes;
+    for (LanguageCache::ConstIterator iter = d->languageCache.constBegin(); iter != d->languageCache.constEnd(); ++iter) {
+        foreach (ILanguage* language, iter.value()) {
+            if (language->name() == languageName) {
+                mimetypes << iter.key();
+                break;
+            }
+        }
+    }
+    return mimetypes;
+}
+
 BackgroundParser *LanguageController::backgroundParser() const
 {
     return d->backgroundParser;
