@@ -61,10 +61,6 @@ Boston, MA 02110-1301, USA.
 #include <sublime/view.h>
 #include <sublime/document.h>
 #include <sublime/urldocument.h>
-#include <language/util/navigationtooltip.h>
-#include <language/duchain/duchainlock.h>
-#include <language/duchain/topducontext.h>
-#include <language/duchain/duchainutils.h>
 
 namespace KDevelop
 {
@@ -122,6 +118,8 @@ MainWindow::MainWindow( Sublime::Controller *parent, Qt::WFlags flags )
 
 MainWindow::~ MainWindow()
 {
+    Core::self()->uiControllerInternal()->mainWindowDeleted(this);
+
     if (memberList().count() == 1) {
         // We're closing down...
         Core::self()->cleanup();
@@ -129,7 +127,6 @@ MainWindow::~ MainWindow()
     }
 
     delete d;
-    Core::self()->uiControllerInternal()->mainWindowDeleted(this);
 }
 
 void MainWindow::ensureVisible()
@@ -305,7 +302,6 @@ void MainWindow::initialize()
     foreach(IPlugin* plugin, Core::self()->pluginController()->loadedPlugins())
         d->addPlugin(plugin);
     
-    guiFactory()->addClient(Core::self()->debugControllerInternal());
     guiFactory()->addClient(Core::self()->sessionController());
     guiFactory()->addClient(Core::self()->sourceFormatterControllerInternal());
     // Needed to re-plug the actions from the sessioncontroller as xmlguiclients don't
@@ -316,14 +312,13 @@ void MainWindow::initialize()
     //Queued so we process it with some delay, to make sure the rest of the UI has already adapted
     connect(Core::self()->documentController(), SIGNAL(documentActivated(KDevelop::IDocument*)), SLOT(updateCaption()), Qt::QueuedConnection);
     connect(Core::self()->documentController(), SIGNAL(documentClosed(KDevelop::IDocument*)), SLOT(updateCaption()), Qt::QueuedConnection);
+    connect(Core::self()->documentController(), SIGNAL(documentUrlChanged(KDevelop::IDocument*)), SLOT(updateCaption()), Qt::QueuedConnection);
     
     connect(Core::self()->projectController(), SIGNAL(projectOpened(KDevelop::IProject*)), SLOT(updateCaption()));
     connect(Core::self()->projectController(), SIGNAL(projectClosed(KDevelop::IProject*)), SLOT(updateCaption()));
 
-    connect(Core::self()->sessionController()->activeSession(), SIGNAL(nameChanged(QString , QString)), SLOT(updateCaption()));
+    connect(Core::self()->sessionController()->activeSession(), SIGNAL(nameChanged(QString,QString)), SLOT(updateCaption()));
     
-    // Remove Handbook entry
-    actionCollection()->action( "help_contents" )->setVisible( false );
     updateCaption();
 }
 
@@ -388,32 +383,9 @@ void MainWindow::tabContextMenuRequested(Sublime::View* view, KMenu* menu)
     d->tabContextMenuRequested(view, menu);
 }
 
-void MainWindow::tabToolTipRequested(Sublime::View* view, QPoint point)
+void MainWindow::tabToolTipRequested(Sublime::View* view, Sublime::Container* container, int tab)
 {
-    Q_UNUSED(point);
-    static QPointer<NavigationToolTip> toolTip;
-    
-    if(toolTip)
-        toolTip->close();
-    
-    DUChainReadLocker lock;
-    
-    Sublime::UrlDocument* urlDoc = dynamic_cast<Sublime::UrlDocument*>(view->document());
-    
-    if(urlDoc) {
-        TopDUContext* top = DUChainUtils::standardContextForUrl(urlDoc->url());
-                
-        if(top)
-        {
-            QWidget* navigationWidget = top->createNavigationWidget();
-            if( navigationWidget )
-            {
-                toolTip = new KDevelop::NavigationToolTip(this, QCursor::pos() + QPoint(20, 20), navigationWidget);
-                toolTip->resize( navigationWidget->sizeHint() + QSize(10, 10) );
-                ActiveToolTip::showToolTip(toolTip);
-            }
-        }
-    }
+    d->tabToolTipRequested(view, container, tab);
 }
 
 void MainWindow::dockBarContextMenuRequested(Qt::DockWidgetArea area, const QPoint& position)
