@@ -1,5 +1,5 @@
 /* This file is part of KDevelop
-Copyright 2008 Anreas Pakulat <apaku@gmx.de>
+Copyright 2008 Andreas Pakulat <apaku@gmx.de>
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
@@ -24,6 +24,7 @@ Boston, MA 02110-1301, USA.
 #include <QtCore/QObject>
 #include <QtCore/QUuid>
 #include <interfaces/isession.h>
+#include <kurl.h>
 
 namespace KDevelop
 {
@@ -33,7 +34,8 @@ class KDEVPLATFORMSHELL_EXPORT Session : public ISession
     Q_OBJECT
 public:
     static const QString cfgSessionNameEntry;
-    Session( const QUuid& );
+    static const QString cfgSessionPrettyContentsEntry;
+    Session( const QUuid&, QObject * parent = 0 );
     virtual ~Session();
 
     virtual KUrl pluginDataArea( const IPlugin* );
@@ -41,9 +43,18 @@ public:
 
     void deleteFromDisk();
 
+    KUrl::List containedProjects() const;
+    
+    void updateDescription();
+    
+    virtual QString description() const;    
     virtual QString name() const;
     void setName( const QString& );
     QUuid id() const;
+
+    virtual void setTemporary(bool temp);
+    virtual bool isTemporary() const;
+
 Q_SIGNALS:
     void nameChanged( const QString& newname, const QString& oldname );
 private:
