@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright 2007 Alexander Dymo  <adymo@kdevelop.org>            *
+ *   Copyright 2007 Alexander Dymo  <adymo@kdevelop.org>                   *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU Library General Public License as       *
@@ -25,7 +25,6 @@
 #include <QWidget>
 #include <QLabel>
 
-
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kconfiggroup.h>
@@ -33,6 +32,7 @@
 #include <kxmlguifactory.h>
 #include <kactioncollection.h>
 #include <kstatusbar.h>
+#include <kdeversion.h>
 
 #include <ktexteditor/view.h>
 #include <ktexteditor/document.h>
@@ -40,28 +40,33 @@
 #include <ktexteditor/codecompletioninterface.h>
 #include <ktexteditor/markinterface.h>
 #include <ktexteditor/configinterface.h>
+#include <ktexteditor/sessionconfiginterface.h>
 
 #include <sublime/area.h>
 #include <sublime/view.h>
 
 #include <interfaces/context.h>
 #include <interfaces/contextmenuextension.h>
+#include <interfaces/ilanguagecontroller.h>
+#include <interfaces/icompletionsettings.h>
+#include <interfaces/iprojectcontroller.h>
+#include <interfaces/iproject.h>
 
 #include <language/interfaces/editorcontext.h>
 
+#include <project/projectutils.h>
+
 #include "core.h"
 #include "mainwindow.h"
 #include "uicontroller.h"
 #include "partcontroller.h"
 #include "plugincontroller.h"
 #include "documentcontroller.h"
-#include <interfaces/ilanguagecontroller.h>
-#include <interfaces/icompletionsettings.h>
-
-#include <kdeversion.h>
 
 namespace KDevelop {
 
+const int MAX_DOC_SETTINGS = 20;
+
 struct TextDocumentPrivate {
     TextDocumentPrivate(TextDocument *textDocument)
         : encoding(""), m_textDocument(textDocument)
@@ -77,13 +82,16 @@ struct TextDocumentPrivate {
             delete m_addedContextMenu;
             m_addedContextMenu = 0;
         }
+        if (document) {
+            saveSessionConfig();
+            delete document;
+        }
     }
 
     QPointer<KTextEditor::Document> document;
     IDocument::DocumentState state;
     QString encoding;
 
-
     void newDocumentStatus(KTextEditor::Document *document)
     {
         bool dirty = (state == IDocument::Dirty || state == IDocument::DirtyAndModified);
@@ -96,7 +104,7 @@ struct TextDocumentPrivate {
         Q_UNUSED(document);
         m_textDocument->notifyContentChanged();
     }
-
+    
     void populateContextMenu( KTextEditor::View* v, QMenu* menu )
     {
         if (m_addedContextMenu) {
@@ -106,12 +114,24 @@ struct TextDocumentPrivate {
             delete m_addedContextMenu;
         }
 
+        m_addedContextMenu = new QMenu();
+
         Context* c = new EditorContext( v, v->cursorPosition() );
         QList<ContextMenuExtension> extensions = Core::self()->pluginController()->queryPluginsForContextMenuExtensions( c );
-        menu->addSeparator();
-
-        m_addedContextMenu = new QMenu();
+        
         ContextMenuExtension::populateMenu(m_addedContextMenu, extensions);
+        
+        {
+            KUrl url = v->document()->url();
+            IProject* project = Core::self()->projectController()->findProjectForUrl( url );
+            if(project)
+            {
+                QList< ProjectBaseItem* > items = project->itemsForUrl( url );
+                if(!items.isEmpty())
+                    populateParentItemsMenu( items.front(), m_addedContextMenu );
+            }
+        }
+        
         foreach ( QAction* action, m_addedContextMenu->actions() ) {
             menu->addAction(action);
         }
@@ -182,6 +202,64 @@ struct TextDocumentPrivate {
         m_textDocument->notifyStateChanged();
     }
 
+    inline KConfigGroup katePartSettingsGroup() const
+    {
+        return KGlobal::config()->group("KatePart Settings");
+    }
+
+    inline QString docConfigGroupName() const
+    {
+        return document->url().pathOrUrl();
+    }
+
+    inline KConfigGroup docConfigGroup() const
+    {
+        return katePartSettingsGroup().group(docConfigGroupName());
+    }
+
+    void saveSessionConfig()
+    {
+        if(!document->url().isValid()) {
+            return;
+        }
+        if (KTextEditor::ParameterizedSessionConfigInterface *sessionConfigIface =
+            qobject_cast<KTextEditor::ParameterizedSessionConfigInterface*>(document))
+        {
+            // make sure only MAX_DOC_SETTINGS entries are stored
+            KConfigGroup katePartSettings = katePartSettingsGroup();
+            // ordered list of documents
+            QStringList documents = katePartSettings.readEntry("documents", QStringList());
+            // ensure this document is "new", i.e. at the end of the list
+            documents.removeOne(docConfigGroupName());
+            documents.append(docConfigGroupName());
+            // remove "old" documents + their group
+            while(documents.size() >= MAX_DOC_SETTINGS) {
+                katePartSettings.group(documents.takeFirst()).deleteGroup();
+            }
+            // update order
+            katePartSettings.writeEntry("documents", documents);
+
+            // actually save session config
+            KConfigGroup group = docConfigGroup();
+            sessionConfigIface->writeParameterizedSessionConfig(group,
+                KTextEditor::ParameterizedSessionConfigInterface::SkipUrl);
+        }
+    }
+
+    void loadSessionConfig()
+    {
+        if (!document || !katePartSettingsGroup().hasGroup(docConfigGroupName())) {
+            return;
+        }
+        if (KTextEditor::ParameterizedSessionConfigInterface *sessionConfigIface =
+            qobject_cast<KTextEditor::ParameterizedSessionConfigInterface*>(document))
+        {
+            sessionConfigIface->readParameterizedSessionConfig(docConfigGroup(),
+                KTextEditor::ParameterizedSessionConfigInterface::SkipUrl);
+        }
+    }
+
+
 private:
     TextDocument *m_textDocument;
     bool m_loaded;
@@ -230,6 +308,14 @@ TextDocument::~TextDocument()
 
 bool TextDocument::isTextDocument() const
 {
+    if( !d->document )
+    {
+        /// @todo Somehow it can happen that d->document is zero, which makes
+        /// code relying on "isTextDocument() == (bool)textDocument()" crash
+        qWarning() << "Broken text-document: " << url();
+        return false;
+    }
+    
     return true;
 }
 
@@ -267,6 +353,8 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
 
         Core::self()->partController()->addPart(d->document, false);
 
+        d->loadSessionConfig();
+
         connect(d->document, SIGNAL(modifiedChanged(KTextEditor::Document*)),
                  this, SLOT(newDocumentStatus(KTextEditor::Document*)));
         connect(d->document, SIGNAL(textChanged(KTextEditor::Document*)),
@@ -275,13 +363,15 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
                  this, SLOT(documentUrlChanged(KTextEditor::Document*)));
         connect(d->document, SIGNAL(documentSavedOrUploaded(KTextEditor::Document*,bool)),
                  this, SLOT(documentSaved(KTextEditor::Document*,bool)));
+        connect(d->document, SIGNAL(marksChanged(KTextEditor::Document*)),
+                 this, SLOT(saveSessionConfig()));
 
         KTextEditor::ModificationInterface *iface = qobject_cast<KTextEditor::ModificationInterface*>(d->document);
         if (iface)
         {
             iface->setModifiedOnDiskWarning(true);
-            connect(d->document, SIGNAL(modifiedOnDisk(KTextEditor::Document*, bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)),
-                this, SLOT(modifiedOnDisk(KTextEditor::Document*, bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)));
+            connect(d->document, SIGNAL(modifiedOnDisk(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)),
+                this, SLOT(modifiedOnDisk(KTextEditor::Document*,bool,KTextEditor::ModificationInterface::ModifiedOnDiskReason)));
         }
 
         notifyTextDocumentCreated();
@@ -293,16 +383,19 @@ QWidget *TextDocument::createViewWidget(QWidget *parent)
         connect(view, SIGNAL(contextMenuAboutToShow(KTextEditor::View*,QMenu*)), this, SLOT(populateContextMenu(KTextEditor::View*,QMenu*)));
 
         //in KDE >= 4.4 we can use KXMLGuiClient::replaceXMLFile to provide
-        //katepart with out own restructured UI configuration
-        QStringList katePartUIs = KGlobal::mainComponent().dirs()->findAllResources("data", "kdevelop/katepartui.rc");
-        const QString katePartUI = katePartUIs.last();
-        const QString katePartLocalUI = KStandardDirs::locateLocal("data", "kdevelop/katepartui.rc");
-        if (!QFile::exists(katePartLocalUI)) {
-            // prevent warning:
-            // kdevelop/kdeui (kdelibs): No such XML file ".../.kde/share/apps/kdevelop/katepartui.rc"
-            QFile::copy(katePartUI, katePartLocalUI);
+        //katepart with our own restructured UI configuration
+        const QString uiFile = KGlobal::mainComponent().componentName() + "/katepartui.rc";
+        QStringList katePartUIs = KGlobal::mainComponent().dirs()->findAllResources("data", uiFile);
+        if (!katePartUIs.isEmpty()) {
+            const QString katePartUI = katePartUIs.last();
+            const QString katePartLocalUI = KStandardDirs::locateLocal("data", uiFile);
+            if (!QFile::exists(katePartLocalUI)) {
+                // prevent warning:
+                // kdevelop/kdeui (kdelibs): No such XML file ".../.kde/share/apps/kdevelop/katepartui.rc"
+                QFile::copy(katePartUI, katePartLocalUI);
+            }
+            view->replaceXMLFile(katePartUI, katePartLocalUI);
         }
-        view->replaceXMLFile(katePartUI, katePartLocalUI);
     }
 
     if (KTextEditor::CodeCompletionInterface* cc = dynamic_cast<KTextEditor::CodeCompletionInterface*>(view))
@@ -502,6 +595,7 @@ bool TextDocument::close(DocumentSaveMode mode)
         return false;
 
     if ( d->document ) {
+        d->saveSessionConfig();
         delete d->document; //We have to delete the document right now, to prevent random crashes in the event handler
     }
 
@@ -655,14 +749,14 @@ void KDevelop::TextEditorWidget::setEditorView(KTextEditor::View* view)
 {
     if (d->view)
     {
-        disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)));
+        disconnect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)));
         removeChildClient(view);
     }
 
     d->view = view;
     insertChildClient(d->view);
-    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), 
-            this, SLOT(viewStatusChanged(KTextEditor::View*, const KTextEditor::Cursor&)));
+    connect(view, SIGNAL(cursorPositionChanged(KTextEditor::View*,KTextEditor::Cursor)), 
+            this, SLOT(viewStatusChanged(KTextEditor::View*,KTextEditor::Cursor)));
     
     viewStatusChanged(view, view->cursorPosition());
 
