@@ -25,6 +25,7 @@
 #include <sublime/controller.h>
 
 #include <ksharedconfig.h>
+#include <interfaces/iassistant.h>
 
 namespace Sublime {
     class AreaIndex;
@@ -36,7 +37,8 @@ namespace KDevelop {
 class Core;
 class MainWindow;
 
-class KDEVPLATFORMSHELL_EXPORT UiController: public Sublime::Controller, public IUiController {
+class KDEVPLATFORMSHELL_EXPORT UiController: public Sublime::Controller, public IUiController
+{
     Q_OBJECT
 
 public:
@@ -62,6 +64,9 @@ public:
     virtual void addToolView(const QString &name, IToolViewFactory *factory);
     virtual void removeToolView(IToolViewFactory *factory);
 
+    virtual QWidget* findToolView(const QString& name, IToolViewFactory *factory, FindFlags flags);
+    virtual void raiseToolView(QWidget* toolViewWidget);
+
     void addNewToolView(MainWindow *mw);
 
     void initialize();
@@ -80,20 +85,35 @@ public:
     /*! @p status must implement KDevelop::IStatus */
     virtual void registerStatus(QObject* status);
 
+    virtual void popUpAssistant(const KDevelop::IAssistant::Ptr& assistant);
+
+    virtual void hideAssistant(const KDevelop::IAssistant::Ptr& assistant);
+
+    virtual void showErrorMessage(const QString& message, int timeout);
+
+    /// Returns list of available view factories together with their ToolDocuments.
+    /// @see addToolView(), removeToolView(), findToolView()
+    const QMap<IToolViewFactory*, Sublime::ToolDocument*>& factoryDocuments() const;
+
+    /// Adds a tool view in the active area to the dock area @p area.
+    /// @see activeArea()
+    void addToolViewToDockArea(const QString& name, IToolViewFactory* factory, Qt::DockWidgetArea area);
+
 private:
     void addToolViewIfWanted(IToolViewFactory* factory,
                            Sublime::ToolDocument* doc,
                            Sublime::Area* area);
-    void addToolViewToArea(IToolViewFactory* factory,
+    Sublime::View* addToolViewToArea(IToolViewFactory* factory,
                            Sublime::ToolDocument* doc,
                            Sublime::Area* area);
-    void saveArea(Sublime::AreaIndex* area, KConfigGroup & group);
-    void loadArea(Sublime::Area* area, Sublime::AreaIndex* areaIndex, const KConfigGroup & group);
-
 
 public Q_SLOTS:
     void raiseToolView(Sublime::View * view);
 
+private Q_SLOTS:
+    void assistantHide();
+    void assistantActionsChanged();
+
 private:
     class UiControllerPrivate* const d;
     friend class UiControllerPrivate;
