@@ -28,7 +28,6 @@
 #include "document.h"
 #include "areaindex.h"
 #include "controller.h"
-#include <qpointer.h>
 
 namespace Sublime {
 
@@ -48,7 +47,7 @@ struct AreaPrivate {
         controller = p.controller;
         toolViewPositions.clear();
         desiredToolViews = p.desiredToolViews;
-        shownToolView = p.shownToolView;
+        shownToolViews = p.shownToolViews;
         workingSet = p.workingSet;
 
         title = p.title;
@@ -90,11 +89,11 @@ struct AreaPrivate {
     QList<View*> toolViews;
     QMap<View *, Sublime::Position> toolViewPositions;
     QMap<QString, Sublime::Position> desiredToolViews;
-    QMap<Sublime::Position, QString> shownToolView;
+    QMap<Sublime::Position, QStringList> shownToolViews;
     QMap<Sublime::Position, int> thickness;
     QString iconName;
     QString workingSet;
-    QPointer<View> activeView;
+    QWeakPointer<View> activeView;
 };
 
 
@@ -129,15 +128,15 @@ Area::Area(const Area &area)
 
 void Area::initialize()
 {
-    connect(this, SIGNAL(viewAdded(Sublime::AreaIndex*, Sublime::View*)),
-            d->controller, SLOT(notifyViewAdded(Sublime::AreaIndex*, Sublime::View*)));
-    connect(this, SIGNAL(aboutToRemoveView(Sublime::AreaIndex*, Sublime::View*)),
-            d->controller, SLOT(notifyViewRemoved(Sublime::AreaIndex*, Sublime::View*)));
-    connect(this, SIGNAL(toolViewAdded(Sublime::View*, Sublime::Position)),
-            d->controller, SLOT(notifyToolViewAdded(Sublime::View*, Sublime::Position)));
-    connect(this, SIGNAL(aboutToRemoveToolView(Sublime::View*, Sublime::Position)),
-            d->controller, SLOT(notifyToolViewRemoved(Sublime::View*, Sublime::Position)));
-    connect(this, SIGNAL(toolViewMoved(Sublime::View*, Sublime::Position)),
+    connect(this, SIGNAL(viewAdded(Sublime::AreaIndex*,Sublime::View*)),
+            d->controller, SLOT(notifyViewAdded(Sublime::AreaIndex*,Sublime::View*)));
+    connect(this, SIGNAL(aboutToRemoveView(Sublime::AreaIndex*,Sublime::View*)),
+            d->controller, SLOT(notifyViewRemoved(Sublime::AreaIndex*,Sublime::View*)));
+    connect(this, SIGNAL(toolViewAdded(Sublime::View*,Sublime::Position)),
+            d->controller, SLOT(notifyToolViewAdded(Sublime::View*,Sublime::Position)));
+    connect(this, SIGNAL(aboutToRemoveToolView(Sublime::View*,Sublime::Position)),
+            d->controller, SLOT(notifyToolViewRemoved(Sublime::View*,Sublime::Position)));
+    connect(this, SIGNAL(toolViewMoved(Sublime::View*,Sublime::Position)),
             d->controller, SIGNAL(toolViewMoved(Sublime::View*)));
 
     /* In theory, ownership is passed to us, so should not bother detecting
@@ -153,7 +152,7 @@ Area::~Area()
 
 View* Area::activeView()
 {
-    return d->activeView;
+    return d->activeView.data();
 }
 
 void Area::setActiveView(View* view)
@@ -161,21 +160,17 @@ void Area::setActiveView(View* view)
     d->activeView = view;
 }
 
-void Sublime::Area::addView(View *view, AreaIndex *index)
+void Area::addView(View *view, AreaIndex *index, View *after)
 {
-    index->add(view);
-    connect(view, SIGNAL(positionChanged(Sublime::View*, int)), this, SLOT(positionChanged(Sublime::View*, int)));
+    //View *after = 0;
+    if (!after  &&  controller()->openAfterCurrent()) {
+        after = activeView();
+    }
+    index->add(view, after);
+    connect(view, SIGNAL(positionChanged(Sublime::View*,int)), this, SLOT(positionChanged(Sublime::View*,int)));
     kDebug() << "view added in" << this;
+    connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
     emit viewAdded(index, view);
-    connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
-}
-
-void Sublime::Area::addViewSilently(View *view, AreaIndex *index)
-{
-    index->add(view);
-    connect(view, SIGNAL(positionChanged(Sublime::View*, int)), this, SLOT(positionChanged(Sublime::View*, int)));
-    kDebug() << "view added in" << this;
-    connect(this, SIGNAL(destroyed()), view, SLOT(deleteLater()));
 }
 
 void Area::addView(View *view, View *after)
@@ -311,10 +306,10 @@ void Area::save(KConfigGroup& group) const
     }
     group.writeEntry("desired views", desired);
     kDebug() << "save " << this << "wrote" << group.readEntry("desired views", "");
-    group.writeEntry("view on left", shownToolView(Sublime::Left));
-    group.writeEntry("view on right", shownToolView(Sublime::Right));
-    group.writeEntry("view on top", shownToolView(Sublime::Top));
-    group.writeEntry("view on bottom", shownToolView(Sublime::Bottom));
+    group.writeEntry("view on left", shownToolViews(Sublime::Left));
+    group.writeEntry("view on right", shownToolViews(Sublime::Right));
+    group.writeEntry("view on top", shownToolViews(Sublime::Top));
+    group.writeEntry("view on bottom", shownToolViews(Sublime::Bottom));
     group.writeEntry("thickness left", thickness(Sublime::Left));
     group.writeEntry("thickness right", thickness(Sublime::Right));
     group.writeEntry("thickness bottom", thickness(Sublime::Bottom));
@@ -341,12 +336,12 @@ void Area::load(const KConfigGroup& group)
             d->desiredToolViews[id] = pos;
         }
     }
-    setShownToolView(Sublime::Left, group.readEntry("view on left", QString()));
-    setShownToolView(Sublime::Right, 
-                     group.readEntry("view on right", QString()));
-    setShownToolView(Sublime::Top, group.readEntry("view on top", QString()));
-    setShownToolView(Sublime::Bottom,
-                     group.readEntry("view on bottom", QString()));
+    setShownToolViews(Sublime::Left, group.readEntry("view on left", QStringList()));
+    setShownToolViews(Sublime::Right,
+                     group.readEntry("view on right", QStringList()));
+    setShownToolViews(Sublime::Top, group.readEntry("view on top", QStringList()));
+    setShownToolViews(Sublime::Bottom,
+                     group.readEntry("view on bottom", QStringList()));
     setThickness(Sublime::Left, group.readEntry("thickness left", -1));
     setThickness(Sublime::Right, group.readEntry("thickness right", -1));
     setThickness(Sublime::Bottom, group.readEntry("thickness bottom", -1));
@@ -360,14 +355,14 @@ bool Area::wantToolView(const QString& id)
     return (d->desiredToolViews.contains(id));
 }
 
-void Area::setShownToolView(Sublime::Position pos, const QString& id)
+void Area::setShownToolViews(Sublime::Position pos, const QStringList& ids)
 {
-    d->shownToolView[pos] = id;
+    d->shownToolViews[pos] = ids;
 }
 
-QString Area::shownToolView(Sublime::Position pos) const
+QStringList Area::shownToolViews(Sublime::Position pos) const
 {
-    return d->shownToolView[pos];
+    return d->shownToolViews[pos];
 }
 
 void Area::setDesiredToolViews(
@@ -400,6 +395,7 @@ void Area::setIconName(const QString& iconName)
 
 void Area::positionChanged(View *view, int newPos)
 {
+    kDebug() << view << newPos;
     AreaIndex *index = indexOf(view);
     index->views().move(index->views().indexOf(view), newPos);
 }
@@ -422,32 +418,56 @@ void Area::setWorkingSet(QString name)
     }
 }
 
-bool Area::closeView(View* view)
+bool Area::closeView(View* view, bool silent)
 {
-    QPointer<Document> doc = view->document();
-                
-    //if(doc && doc->views().count() == 1) {    
-    if (doc && doc->uniqueView(this, view)) {
-        if(!doc->closeDocument())
-            return false;
-        else
-            return true;
-    }
-    
-    //close only one active view
-    removeView(view);
-    delete view;
+    QWeakPointer<Document> doc = view->document();
 
-    if(doc && doc->views().count() == 0)
-        doc->closeDocument(); //close the document instead
+    // We don't just delete the view, because if silent is false, we might need to ask the user.
+    if(doc && !silent)
+    {
+        // Do some counting to check whether we need to ask the user for feedback
+        kDebug() << "Closing view for" << view->document()->documentSpecifier() << "views" << view->document()->views().size() << "in area" << this;
+        int viewsInCurrentArea = 0; // Number of views for the same document in the current area
+        int viewsInOtherAreas = 0; // Number of views for the same document in other areas
+        int viewsInOtherWorkingSets = 0; // Number of views for the same document in areas with different working-set
+
+        foreach(View* otherView, doc.data()->views())
+        {
+            Area* area = controller()->areaForView(otherView);
+            if(area == this)
+                viewsInCurrentArea += 1;
+            if(!area || (area != this))
+                viewsInOtherAreas += 1;
+            if(area && area != this && area->workingSet() != workingSet())
+                viewsInOtherWorkingSets += 1;
+        }
+
+        if(viewsInCurrentArea == 1 && (viewsInOtherAreas == 0 || viewsInOtherWorkingSets == 0))
+        {
+            // Time to ask the user for feedback, because the document will be completely closed
+            // due to working-set synchronization
+            if( !doc.data()->askForCloseFeedback() )
+                return false;
+        }
+    }
+
+    // otherwise we can silently close the view,
+    // the document will still have an opened view somewhere
+    AreaIndex *index = indexOf(view);
+    Q_ASSERT(index);
+
+    emit aboutToRemoveView(index, view);
+    index->remove(view);
+    emit viewRemoved(index, view);
+    delete view;
 
     return true;
 }
 
-void Area::clearViews()
+void Area::clearViews(bool silent)
 {
     foreach(Sublime::View* view, views())
-        closeView(view);
+        closeView(view, silent);
 }
 
 }
